"""
/***************************************************************************
Name			 	 : Geospatial Simulation
Description          : Geospatial tool for managing point-based simulation models
Date                 : 05/Dec/11 
copyright            : (C) 2011 by Dr. Kelly Thorp, USDA-ARS
email                : kelly.thorp@ars.usda.gov 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import QObject, SIGNAL
from PyQt4.QtGui import QAction, QIcon, QMessageBox
# Initialize Qt resources from file resources.py
import os
import sys
import resources
# Import the code for the dialogs
from GeoprocessorDlg import GeoprocessorDlg
from Raster2VectorDlg import Raster2VectorDlg
from ControlFileDlg import ControlFileDlg
from SimControllerDlg import SimControllerDlg
from OptimizationFileDlg import OptimizationFileDlg
from SimOptimizerDlg import SimOptimizerDlg

class GeospatialSimulation: 

    def __init__(self, iface):
        # Save reference to the QGIS interface
        self.iface = iface

    def initGui(self):  
        # Create action that will start plugin configuration
        icon = QIcon(":/plugins/geospatialsimulation/icon.png")
        self.ras2vec = QAction(icon,"Raster to Vector Converter", self.iface.mainWindow())
        self.geoprocess = QAction(icon,"Vector Geoprocessor", self.iface.mainWindow())
        self.controlfile = QAction(icon,"Control File Creator", self.iface.mainWindow())
        self.simulate = QAction(icon,"Simulation Controller", self.iface.mainWindow())
        self.optfile = QAction(icon, "Optimization File Creator", self.iface.mainWindow())
        self.optimize = QAction(icon, "Simulation Optimizer", self.iface.mainWindow())
        self.helpme = QAction(icon, "Help", self.iface.mainWindow())
        
        # connect the action to a method
        QObject.connect(self.ras2vec, SIGNAL("activated()"), self.Raster2Vector)
        QObject.connect(self.geoprocess, SIGNAL("activated()"), self.Geoprocess)
        QObject.connect(self.controlfile, SIGNAL("activated()"), self.MakeControlFile)
        QObject.connect(self.simulate, SIGNAL("activated()"), self.Simulate)
        QObject.connect(self.optfile, SIGNAL("activated()"), self.MakeOptimizationFile)
        QObject.connect(self.optimize, SIGNAL("activated()"), self.Optimize)
        QObject.connect(self.helpme, SIGNAL("activated()"), self.Help)
         
        # Add toolbar button and menu item        
        self.iface.addPluginToMenu("&Geospatial Simulation", self.ras2vec)
        self.iface.addPluginToMenu("&Geospatial Simulation", self.geoprocess)
        self.iface.addPluginToMenu("&Geospatial Simulation", self.controlfile)
        self.iface.addPluginToMenu("&Geospatial Simulation", self.simulate)
        self.iface.addPluginToMenu("&Geospatial Simulation", self.optfile)
        self.iface.addPluginToMenu("&Geospatial Simulation", self.optimize)
        self.iface.addPluginToMenu("&Geospatial Simulation", self.helpme)
        

    def unload(self):
        # Remove the plugin menu item and icon
        self.iface.removePluginMenu("&Geospatial Simulation", self.ras2vec)
        self.iface.removePluginMenu("&Geospatial Simulation", self.geoprocess)
        self.iface.removePluginMenu("&Geospatial Simulation", self.controlfile)
        self.iface.removePluginMenu("&Geospatial Simulation", self.simulate)
        self.iface.removePluginMenu("&Geospatial Simulation", self.optfile)
        self.iface.removePluginMenu("&Geospatial Simulation", self.optimize)
        self.iface.removePluginMenu("&Geospatial Simulation", self.helpme)
        
    # run methods that perform all the real work
    def Geoprocess(self): 
        # create and show the dialog 
        dlg = GeoprocessorDlg(self.iface) 
        # show the dialog
        #dlg.show() #Modeless dialog
        dlg.exec_() #Modal dialog
        
    def Raster2Vector(self):
        dlg = Raster2VectorDlg(self.iface)
        dlg.exec_()
        
    def MakeControlFile(self):
        dlg = ControlFileDlg(self.iface)
        dlg.exec_()
    
    def Simulate(self):
        dlg = SimControllerDlg(self.iface)
        dlg.exec_()
        
    def MakeOptimizationFile(self):
        dlg = OptimizationFileDlg(self.iface)
        dlg.exec_()
    
    def Optimize(self):
        dlg = SimOptimizerDlg(self.iface)
        dlg.exec_()
        
    def Help(self):
        path = os.path.dirname(sys.modules[__name__].__file__)
        if sys.platform == 'linux':
            os.system(path+"//Geospatial Simulation README.pdf")
        elif sys.platform == 'win32':
            os.startfile(path+"\\Geospatial Simulation README.pdf")
        else:
            QMessageBox.critical(self.iface.mainWindow(),'Help','Error opening document. Look in plug-in install directory for PDF.')